/* Sistemas Integrados TECNISA
   Servios

Nome: TAREFA_POR_FUNCIONARIO

Descrio:
Clculo das datas e horas previstas para incio e concluso das tarefas de um
funcionrio.

P_CDFILIAL   - Cdigo da Filial
P_CDOPERRESP - Cdigo do OPERADOR

Para aplicar e debugar
-- @H:\SISTEMAS\DBA\SQL\ORACLE\7200_SRVSTP_TAREFA_POR_FUNCIONARIO.SQL;
-- EXECUTE TAREFA_POR_FUNCIONARIO('9999', '01', '0000000002')
-- SET SERVEROUTPUT ON
-- DBMS_OUTPUT.PUT_LINE('xx');

Criao:
11/12/99 - Ederson

*/

CREATE OR REPLACE
PROCEDURE TAREFA_POR_FUNCIONARIO
  (P_CDFILIAL   IN VARCHAR2,
   P_CDOPERRESP IN VARCHAR2) IS

   /* - Variveis utilizadas para receber valores do cursor ----------------- */
   VC_IDSTATUSTARE VARCHAR2(01);
   VC_NRPRIOSOLI   VARCHAR2(03);
   VC_NRPRIOTARE   VARCHAR2(03);
   VC_CDFILIAL     VARCHAR2(04);
   VC_NRSOLISERV   VARCHAR2(10);
   VC_NRTARESERV   VARCHAR2(10);
   VC_DTPREVINITAR DATE;
   VC_DTPREVENCTAR DATE;
   VC_DTINICTARE   DATE;
   VC_QTHORASTARE  NUMBER(16, 02);
   /* ----------------------------------------------------------------------- */

   V_IDPRIMEIRA  VARCHAR2(1);

   V_DTHRINICIAL DATE;

   V_DTHRINICALC DATE;
   V_DTHRFINCALC DATE;

   /* - Declarao de cursores ---------------------------------------------- */

   CURSOR C_TAREFAS_FUNC_FILA IS
      SELECT
         TA.IDSTATUSTARE,
         SO.NRPRIOSOLI,
         TA.NRPRIOTARE,
         TA.CDFILIAL,
         TA.NRSOLISERV,
         TA.NRTARESERV,
         TA.DTPREVINITAR,
         TA.DTPREVENCTAR,
         TA.DTINICTARE,
         TA.QTHORASTARE
      FROM
         TARESERV TA,
         SOLISERV SO
      WHERE
         (TA.CDFILIAL   = P_CDFILIAL)    AND
         (TA.CDOPERRESP = P_CDOPERRESP)  AND
         (SO.CDFILIAL   = TA.CDFILIAL)   AND
         (SO.NRSOLISERV = TA.NRSOLISERV) AND
         (TA.IDSTATUSTARE IN ('A', 'P'))
      ORDER BY
         TA.IDSTATUSTARE,
         SO.NRPRIOSOLI,
         TA.NRPRIOTARE,
         TA.CDFILIAL,
         TA.NRTARESERV;

BEGIN
   V_IDPRIMEIRA := 'S';

   OPEN C_TAREFAS_FUNC_FILA;
   LOOP
      FETCH C_TAREFAS_FUNC_FILA
      INTO VC_IDSTATUSTARE,
           VC_NRPRIOSOLI,
           VC_NRPRIOTARE,
           VC_CDFILIAL,
           VC_NRSOLISERV,
           VC_NRTARESERV,
           VC_DTPREVINITAR,
           VC_DTPREVENCTAR,
           VC_DTINICTARE,
           VC_QTHORASTARE;
      EXIT WHEN C_TAREFAS_FUNC_FILA%NOTFOUND;

      IF VC_NRPRIOSOLI <> '000' AND VC_NRPRIOSOLI <> '999' THEN
         /* Somente tarefas com a fila da solicitao diferente de 000 ou 999 */

         IF V_IDPRIMEIRA = 'S' THEN

            IF VC_IDSTATUSTARE = 'A' THEN /* - Tarefa em Andamento ---------- */
                                          /* Data inicial = data/hora real de
                                             incio da tarefa                 */
               V_DTHRINICIAL := VC_DTINICTARE;
            ELSE                          /* - Tarefa Pendente -------------- */
                                          /* Data inicial = data/hora atual   */
               V_DTHRINICIAL := TO_DATE(TO_CHAR(SYSDATE, 'YYYYMMDD HH24MI'), 'YYYYMMDD HH24MI');
            END IF;

            /* Considera o prximo incio de expediente ----------------------*/
            V_DTHRINICIAL := AJUSTA_EXPEDIENTE(P_CDFILIAL,
                                               P_CDOPERRESP,
                                               V_DTHRINICIAL,
                                               'CONSIDERA_LIMITE');
         END IF;

         IF VC_IDSTATUSTARE = 'A' THEN
            /* ############################################################## */
            /* # 1 CASO: Tarefa em andamento ############################### */
            /* ############################################################## */

            CALCULA_DATAS_TAREFA(P_CDFILIAL,
                                 P_CDOPERRESP,
                                 V_DTHRINICIAL,
                                 VC_QTHORASTARE,
                                 V_DTHRINICALC,
                                 V_DTHRFINCALC);

            /* Mantm na tarefa as datas/horas que j estavam previstas
               (calculadas anteriormente). S atualiza a data/hora de fim real
               que  calculada tomando como base a data/hora de incio real e
               quantidade de horas para realizao da tarefa.
               Na concluso da tarefa o sistema atualiza corretamente a
               data/hora real de concluso.                                   */

            IF VC_DTPREVINITAR IS NULL AND VC_DTPREVENCTAR IS NULL THEN
               VC_DTPREVINITAR := V_DTHRINICIAL;
               VC_DTPREVENCTAR := V_DTHRFINCALC;
            END IF;
            UPDATE
               TARESERV
            SET
               DTPREVINITAR = VC_DTPREVINITAR,
               DTPREVENCTAR = VC_DTPREVENCTAR,
               DTENCETARE   = V_DTHRFINCALC
            WHERE
               (CDFILIAL   = VC_CDFILIAL)   AND
               (NRSOLISERV = VC_NRSOLISERV) AND
               (NRTARESERV = VC_NRTARESERV);

            /* Atualiza a solicitao pesquisando nas suas tarefas            */
            UPDATE
               SOLISERV
            SET
               DTPREVINISOL = (SELECT
                                  MIN(DTPREVINITAR)
                               FROM
                                  TARESERV
                               WHERE
                                  (CDFILIAL   = VC_CDFILIAL) AND
                                  (NRSOLISERV = VC_NRSOLISERV)),
               DTPREVENCSOL = (SELECT
                                  MAX(DTPREVENCTAR)
                               FROM
                                  TARESERV
                               WHERE
                                  (CDFILIAL   = VC_CDFILIAL) AND
                                  (NRSOLISERV = VC_NRSOLISERV)),
               DTINICSOLI   = (SELECT
                                  MIN(DTINICTARE)
                               FROM
                                  TARESERV
                               WHERE
                                  (CDFILIAL   = VC_CDFILIAL) AND
                                  (NRSOLISERV = VC_NRSOLISERV)),
               DTENCESOLI   = (SELECT
                                  MAX(DTENCETARE)
                               FROM
                                  TARESERV
                               WHERE
                                  (CDFILIAL   = VC_CDFILIAL) AND
                                  (NRSOLISERV = VC_NRSOLISERV))
            WHERE
               (CDFILIAL   = VC_CDFILIAL) AND
               (NRSOLISERV = VC_NRSOLISERV);

            /* Para a prxima tarefa soma 1 min na data de previso anterior  */
            V_DTHRINICIAL := V_DTHRFINCALC + 0.0006944444;
                                                         /* 0.0006944 = 1 min */
            V_DTHRINICIAL := AJUSTA_EXPEDIENTE(P_CDFILIAL,
                                               P_CDOPERRESP,
                                               V_DTHRINICIAL,
                                               'CONSIDERA_LIMITE');
         ELSE                         /* - Tarefa Pendente ------------------ */
            IF VC_QTHORASTARE = 0.00 THEN
               /* ########################################################### */
               /* # 2 CASO: Tarefa pendente, Qtde.Horas = 0 ################ */
               /* ########################################################### */

               /* - Tarefa ainda no dimensionada --------------------------- */
               /* Deixa as datas vazias                                       */

               UPDATE
                  TARESERV
               SET
                  DTPREVINITAR = NULL,
                  DTPREVENCTAR = NULL,
                  DTINICTARE   = NULL,
                  DTENCETARE   = NULL
               WHERE
                  (CDFILIAL   = VC_CDFILIAL)   AND
                  (NRSOLISERV = VC_NRSOLISERV) AND
                  (NRTARESERV = VC_NRTARESERV);

               /* Atualiza a solicitao pesquisando nas outras tarefas       */
               UPDATE
                  SOLISERV
               SET
                  DTPREVINISOL = (SELECT
                                     MIN(DTPREVINITAR)
                                  FROM
                                     TARESERV
                                  WHERE
                                     (CDFILIAL   =  VC_CDFILIAL)   AND
                                     (NRSOLISERV =  VC_NRSOLISERV) AND
                                     (NRTARESERV <> VC_NRTARESERV)),
                  DTPREVENCSOL = (SELECT
                                     MAX(DTPREVENCTAR)
                                  FROM
                                     TARESERV
                                  WHERE
                                     (CDFILIAL   =  VC_CDFILIAL)   AND
                                     (NRSOLISERV =  VC_NRSOLISERV) AND
                                     (NRTARESERV <> VC_NRTARESERV)),
                  DTINICSOLI   = (SELECT
                                     MIN(DTINICTARE)
                                  FROM
                                     TARESERV
                                  WHERE
                                     (CDFILIAL   = VC_CDFILIAL) AND
                                     (NRSOLISERV = VC_NRSOLISERV)),
                  DTENCESOLI   = (SELECT
                                     MAX(DTENCETARE)
                                  FROM
                                     TARESERV
                                  WHERE
                                     (CDFILIAL   = VC_CDFILIAL) AND
                                     (NRSOLISERV = VC_NRSOLISERV))
               WHERE
                  (CDFILIAL   = VC_CDFILIAL) AND
                  (NRSOLISERV = VC_NRSOLISERV);

               /* Para a prxima tarefa mantm a data anterior                */
               V_DTHRINICIAL := V_DTHRINICIAL;
            ELSE
               IF P_CDOPERRESP IS NULL THEN
                  /* ######################################################## */
                  /* # 3 CASO: Tarefa pendente, Qtde.Horas <> 0, Func = NULL */
                  /* ######################################################## */

                  /* Se o funcionrio estiver vazio atualiza a tarefa com as
                     mesmas datas de previso da solicitao                  */

                  SELECT
                     DTPREVINISOL,
                     DTPREVENCSOL
                  INTO
                     VC_DTPREVINITAR,
                     VC_DTPREVENCTAR
                  FROM
                     SOLISERV
                  WHERE
                     (CDFILIAL   = VC_CDFILIAL) AND
                     (NRSOLISERV = VC_NRSOLISERV);

                  UPDATE
                     TARESERV
                  SET
                     DTPREVINITAR = VC_DTPREVINITAR,
                     DTPREVENCTAR = VC_DTPREVENCTAR,
                     DTINICTARE   = NULL,
                     DTENCETARE   = NULL
                  WHERE
                     (CDFILIAL   = VC_CDFILIAL)   AND
                     (NRSOLISERV = VC_NRSOLISERV) AND
                     (NRTARESERV = VC_NRTARESERV);

                  /* Para a prxima tarefa mantm a data anterior             */
                  V_DTHRINICIAL := V_DTHRINICIAL;
               ELSE
                  /* ######################################################## */
                  /* # 4 CASO: Tarefa pendente, Qtde.Horas <> 0,Func <> NULL */
                  /* ######################################################## */

                  /* Se o funcionrio no estiver vazio calcula normalmente   */

                  CALCULA_DATAS_TAREFA(P_CDFILIAL,
                                       P_CDOPERRESP,
                                       V_DTHRINICIAL,
                                       VC_QTHORASTARE,
                                       V_DTHRINICALC,
                                       V_DTHRFINCALC);

                  /* Atualiza a tarefa com as datas calculadas                */
                  UPDATE
                     TARESERV
                  SET
                     DTPREVINITAR = V_DTHRINICALC,
                     DTPREVENCTAR = V_DTHRFINCALC,
                     DTINICTARE   = NULL,
                     DTENCETARE   = NULL
                  WHERE
                     (CDFILIAL   = VC_CDFILIAL)   AND
                     (NRSOLISERV = VC_NRSOLISERV) AND
                     (NRTARESERV = VC_NRTARESERV);

                  /* Atualiza a solicitao pesquisando nas suas tarefas      */
                  UPDATE
                     SOLISERV
                  SET
                     DTPREVINISOL = (SELECT
                                        MIN(DTPREVINITAR)
                                     FROM
                                        TARESERV
                                     WHERE
                                        (CDFILIAL   =  VC_CDFILIAL)   AND
                                        (NRSOLISERV =  VC_NRSOLISERV)),
                     DTPREVENCSOL = (SELECT
                                        MAX(DTPREVENCTAR)
                                     FROM
                                        TARESERV
                                     WHERE
                                        (CDFILIAL   =  VC_CDFILIAL)   AND
                                        (NRSOLISERV =  VC_NRSOLISERV)),
                     DTINICSOLI   = (SELECT
                                        MIN(DTINICTARE)
                                     FROM
                                        TARESERV
                                     WHERE
                                        (CDFILIAL   = VC_CDFILIAL) AND
                                        (NRSOLISERV = VC_NRSOLISERV)),
                     DTENCESOLI   = (SELECT
                                        MAX(DTENCETARE)
                                     FROM
                                        TARESERV
                                     WHERE
                                        (CDFILIAL   = VC_CDFILIAL) AND
                                        (NRSOLISERV = VC_NRSOLISERV))
                  WHERE
                     (CDFILIAL   = VC_CDFILIAL) AND
                     (NRSOLISERV = VC_NRSOLISERV);
                  /* Para a prxima tarefa soma 1 min na data de previso
                     anterior                                                 */
                  V_DTHRINICIAL := V_DTHRFINCALC + 0.0006944444;
                                                      /* 0.0006944444 = 1 min */
                  V_DTHRINICIAL := AJUSTA_EXPEDIENTE(P_CDFILIAL,
                                                     P_CDOPERRESP,
                                                     V_DTHRINICIAL,
                                                     'CONSIDERA_LIMITE');
               END IF; /* Teste de funcionrio com cdigo vazio */
            END IF; /* Teste de quantidade de horas da tarefa */
         END IF; /* Teste de tarefa em andamento ou pendente */

         V_IDPRIMEIRA := 'N';
      ELSE
         IF VC_IDSTATUSTARE = 'P' THEN
            UPDATE
               TARESERV
            SET
               DTPREVINITAR = NULL,
               DTPREVENCTAR = NULL
            WHERE
               (CDFILIAL   = VC_CDFILIAL)   AND
               (NRSOLISERV = VC_NRSOLISERV) AND
               (NRTARESERV = VC_NRTARESERV);
         END IF;
         UPDATE
            SOLISERV
         SET
            DTPREVINISOL = NULL,
            DTPREVENCSOL = NULL
         WHERE
            (CDFILIAL   = VC_CDFILIAL) AND
            (NRSOLISERV = VC_NRSOLISERV);
      END IF; /* Teste de fila da solicitao diferente de 000 ou 999 */
   END LOOP;
   CLOSE C_TAREFAS_FUNC_FILA;
END;
/
