/* Sistemas Integrados TECNISA

Nome: 5060_FISTRGO.SQL

Descrio:
   Trigger para gravar o log das alteraes

Criao:
   4/12/2012 - Marcelo Alexsandro

Tabela MSGVINCULOREGRA: Regra Fiscal.
Descrio: A trigger insere registros na tabela LOGREGRAFISCAL a partir de alteraes na tabela MSGVINCULOREGRA
*/

IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'LOG_FIS_MSGVINCULOREGRA' AND
                 TYPE   = 'TR' 	           AND
	              U.UID  = O.UID             AND
	              U.NAME = USER)
DROP TRIGGER LOG_FIS_MSGVINCULOREGRA
GO

CREATE TRIGGER LOG_FIS_MSGVINCULOREGRA
ON MSGVINCULOREGRA
FOR INSERT, UPDATE, DELETE
AS
   DECLARE @V_USUARIO    VARCHAR(40);
   DECLARE @V_DTOPER     DATETIME;

   DECLARE @V_ID_COMANDO VARCHAR(1);
   DECLARE @V_DESC       VARCHAR(350);
   DECLARE @V_ITPESQ     VARCHAR(40);
   DECLARE @V_NMOPERADOR VARCHAR(40);
   DECLARE @V_IDTIPOOPER VARCHAR(1);

   DECLARE @V_CDREGRAFISC_OLD     VARCHAR(10)
   DECLARE @V_NRVINCFISCREGRA_OLD VARCHAR(10)
   DECLARE @V_NRMENSREGRAFISC_OLD VARCHAR(10)
   DECLARE @V_DSMENSREGRAFISC_OLD VARCHAR(150)

   DECLARE @V_CDREGRAFISC_NEW     VARCHAR(10)
   DECLARE @V_NRVINCFISCREGRA_NEW VARCHAR(10)
   DECLARE @V_NRMENSREGRAFISC_NEW VARCHAR(10)
   DECLARE @V_DSMENSREGRAFISC_NEW VARCHAR(150)

BEGIN
/* IDTIPOOPER - '1' - Incluso, '2' - Alterao, '3' - Excluso*/
   SET @V_DESC       = '';
   SET @V_ITPESQ     = '';
   SET @V_NMOPERADOR = '';
   SET @V_IDTIPOOPER = '';

   EXEC NMUSUARIO @V_NMOPERADOR;
   SET @V_NMOPERADOR = ISNULL(@V_NMOPERADOR, '');

   DECLARE C_CURSOR CURSOR FOR
   SELECT
     'D' IDCOMANDO,
     DELETED.CDREGRAFISC,
     DELETED.NRVINCFISCREGRA,
     DELETED.NRMENSREGRAFISC,
     DELETED.DSMENSREGRAFISC,
     NULL,
     NULL,
     NULL,
     NULL
   FROM
     DELETED
   WHERE
     CDREGRAFISC+NRVINCFISCREGRA+NRMENSREGRAFISC NOT IN (SELECT CDREGRAFISC+NRVINCFISCREGRA+NRMENSREGRAFISC FROM INSERTED)

   UNION ALL

   SELECT
     'U' IDCOMANDO,
     DELETED.CDREGRAFISC,
     DELETED.NRVINCFISCREGRA,
     DELETED.NRMENSREGRAFISC,
     DELETED.DSMENSREGRAFISC,
     INSERTED.CDREGRAFISC,
     INSERTED.NRVINCFISCREGRA,
     INSERTED.NRMENSREGRAFISC,
     INSERTED.DSMENSREGRAFISC
   FROM
      INSERTED,
      DELETED
   WHERE INSERTED.CDREGRAFISC = DELETED.CDREGRAFISC
     AND INSERTED.NRMENSREGRAFISC = DELETED.NRMENSREGRAFISC
     AND INSERTED.NRVINCFISCREGRA = DELETED.NRVINCFISCREGRA

   UNION ALL

   SELECT
     'I' IDCOMANDO,
     NULL,
     NULL,
     NULL,
     NULL,
     INSERTED.CDREGRAFISC,
     INSERTED.NRVINCFISCREGRA,
     INSERTED.NRMENSREGRAFISC,
     INSERTED.DSMENSREGRAFISC
   FROM INSERTED
   WHERE CDREGRAFISC+NRVINCFISCREGRA+NRMENSREGRAFISC NOT IN (SELECT CDREGRAFISC+NRVINCFISCREGRA+NRMENSREGRAFISC FROM DELETED);
   OPEN C_CURSOR;
   FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,
                                 @V_CDREGRAFISC_OLD,
                                 @V_NRVINCFISCREGRA_OLD,
                                 @V_NRMENSREGRAFISC_OLD,
                                 @V_DSMENSREGRAFISC_OLD,
                                 @V_CDREGRAFISC_NEW,
                                 @V_NRVINCFISCREGRA_NEW,
                                 @V_NRMENSREGRAFISC_NEW,
                                 @V_DSMENSREGRAFISC_NEW;
   WHILE (@@FETCH_STATUS = 0) BEGIN
      IF @V_ID_COMANDO = 'U' BEGIN
         SET @V_DESC   = 'Mensagem = ' + @V_DSMENSREGRAFISC_NEW;
         SET @V_ITPESQ = @V_CDREGRAFISC_NEW+@V_NRVINCFISCREGRA_NEW+@V_NRMENSREGRAFISC_NEW;
         SET @V_IDTIPOOPER = '2';
      END
      ELSE BEGIN
         IF @V_ID_COMANDO = 'I' BEGIN
            SET @V_DESC   = 'Mensagem = ' + @V_DSMENSREGRAFISC_NEW;
            SET @V_ITPESQ = @V_CDREGRAFISC_NEW+@V_NRVINCFISCREGRA_NEW+@V_NRMENSREGRAFISC_NEW;
            SET @V_IDTIPOOPER = '1';
         END
         ELSE BEGIN
            IF @V_ID_COMANDO = 'D' BEGIN
               SET @V_DESC   = 'Mensagem = ' + @V_DSMENSREGRAFISC_OLD;
               SET @V_ITPESQ = @V_CDREGRAFISC_OLD+@V_NRVINCFISCREGRA_OLD+@V_NRMENSREGRAFISC_OLD;
               SET @V_IDTIPOOPER = '3';
            END;
         END;
      END;
      INSERT INTO LOGREGRAFISCAL
         (NMTABELA,
          DTOPERACAO,
          NMRESPOPER,
          IDTIPOOPER,
          DSOPERACAO,
          CDPESQITEM)
       VALUES
         ('MSGVINCULOREGRA',
          GETDATE(),
          ISNULL(@V_NMOPERADOR,' '),
          @V_IDTIPOOPER,
          @V_DESC,
          @V_ITPESQ);
      FETCH NEXT FROM C_CURSOR INTO @V_ID_COMANDO,
                                    @V_CDREGRAFISC_OLD,
                                    @V_NRVINCFISCREGRA_OLD,
                                    @V_NRMENSREGRAFISC_OLD,
                                    @V_DSMENSREGRAFISC_OLD,
                                    @V_CDREGRAFISC_NEW,
                                    @V_NRVINCFISCREGRA_NEW,
                                    @V_NRMENSREGRAFISC_NEW,
                                    @V_DSMENSREGRAFISC_NEW;
   END;
   CLOSE C_CURSOR;
   DEALLOCATE C_CURSOR;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'LOG_FIS_MSGVINCULOREGRA','5.30.xxx';
GO
